LIBNAME library 'C:\nhtsa\NASS\2011\20121218';
LIBNAME nass2011 'C:\nhtsa\NASS\2011\20121218';
/***********************************************************************************************
*   
*   Prepare accident, vehicle, and event data for scenarios
*
************************************************************************************************/

* 1 - Merge Accident/Vehicle Data;
data acc_gv;
	merge nass2011.accident nass2011.gv;
	by psu caseno;
run;
data gv_ve;
	merge acc_gv nass2011.ve;
	by psu caseno vehno;
run;

* 2 - Determine vehicles involved in first event and first object contacted;
proc sort data=nass2011.event out=junk;
	by psu caseno vehnum accseq;
run;
data e_firstveh; * First Vehicle-to-Vehicle Crash;
	set junk;
	by psu caseno vehnum accseq;

	retain first_vehaccseq first_vehs; * keep these for every event for each case;
	if (first.caseno) then do;
		first_vehaccseq = 99; * accident sequence of first vehicle-to-vehicle crash;
		first_vehs = '99';
	end;

	if (1<=objcont<=30)	AND (accseq<first_vehaccseq) then do; * this event is a vehicle-to-vehicle crash;
		first_vehaccseq = accseq;
		dum = vehnum*10 + objcont;
		dumm = put(dum, $8. -L); * convert to string;
		first_vehs = dumm;
	end;

	if (last.caseno); * keep 1 record per crash only;

	keep psu caseno first_vehaccseq first_vehs;
run;
data e_merge; * merge first_vehs back with event table;
	merge e_firstveh nass2011.event;
	by psu caseno;
run;

data e1; * first vehicle;
	set e_merge;
	vehno = vehnum;
	keep psu caseno accseq vehno objcont class1 class2 ratwgt 
		gadev1 gadev2 first_vehaccseq first_vehs;
run;
data e2; * second vehicle;
	set e_merge;
	if class2 ne 0;
	length gump $ 1;
	goon = vehnum;
	vehno = objcont;
	objcont = goon;
	goon = class1;
	class1 = class2;
	class2 = goon;
	gump = gadev2;
	gadev2 = gadev1;
	gadev1 = gump;
	keep psu caseno accseq vehno objcont class1 class2 ratwgt 
		gadev1 gadev2 first_vehaccseq first_vehs;
run;
data ecombo; * put together;
	set e1 e2;
run;
proc sort data=ecombo out = esort;
	by psu caseno vehno accseq;
run;
data e; * determine first harmful event;
	format e1 objcont.;
	set esort;
	by psu caseno vehno accseq;

	e1 = objcont; * store object;
	if (first.vehno); * first event for this vehicle;
	
	keep psu caseno vehno ratwgt e1 first_vehs;
run;

data vehicle_super; * vehicle level table with variables needed for scenarios;
	merge e gv_ve;
	by psu caseno vehno;

	* note: only keeping these variables necessary for scenarios.
	you make keep more variables here for future analysis;
	keep psu caseno vehno vehforms ratwgt
		first_vehs e1 objcont1
		preevent premove acctype;
run;

* Check: Number of crashes with acctype = 1;
title1 'Should be 161339.08';
proc tabulate data=vehicle_super;
	var ratwgt;
	tables ratwgt;
	where acctype = 1;
run;
title1 'The SAS System';
