/************************************************************************************
*
*	Define road departure scenarios
*
*	rev0 - 8/13/13 - Kristofer D. Kusano
*************************************************************************************/
* format for detailed crash scenarios;
proc format;
VALUE  veh_scen
	0 = 'Unknown Scenario'
	1 = 'Single Veh - lane keeping'
	2 = 'Single Veh - lane change'
	15 = 'Single Veh - turning'
	12 = 'Single Veh - avoidance maneuver'
	3 = 'Single Veh - maneuver unknown'
	4 = 'Single Veh - backing'
	5 = 'Single Veh - veh failure'
	6 = 'Single Veh - control loss, action'
	7 = 'Single Veh - control loss, no action'
	8 = 'Single Veh - control loss, backing'
	9 = 'Single Veh - control loss, unkown maneuver'
	16 = 'Single Veh - Pedestrian in Road'
	17 = 'Single Veh - Cyclist in Road'
	18 = 'Single Veh - Object in Road'
	19 = 'Single Veh - Animal in Road'
	20 = 'Single Veh - Train'
	11 = 'Single Veh - Other'
	23 = 'Mult Veh - same dir, avoidance maneuver'
	24 = 'Mult Veh - same dir, lane keeping'
	25 = 'Mult Veh - same dir, lane change'
	27 = 'Mult Veh - same dir, other/unknown maneuver'
	29 = 'Mult Veh - U-turn'
	31 = 'Mult Veh - parked car, unknown maneuver'
	32 = 'Mult Veh - parked car, lane keeping'
	33 = 'Mult Veh - parked car, turning'
	34 = 'Mult Veh - parked car, avoidance'
	35 = 'Mult Veh - parked car, lane change'
	36 = 'Mult Veh - OD, depart lane, lane keeping'
	37 = 'Mult Veh - OD, depart lane, avoidance maneuver'
	38 = 'Mult Veh - OD, depart lane, lane change/merge/overtake'
	39 = 'Mult Veh - OD, depart lane, other maneuver'
	40 = 'Mult Veh - control loss, no veh action'
	41 = 'Mult Veh - control loss, veh action'
	42 = 'Mult Veh - control loss, unknown action'
	50 = 'Mult Veh - Roadside object into Vehicle, lane keeping'
	51 = 'Mult Veh - Roadside object into Vehicle, maneuver'
	69 = 'Mult Veh - Rear End Maneuver'
	70 = 'Mult Veh - Rear End No Maneuver'
	71 = 'Mult Veh - Crossing Path, Both Straight'
	73 = 'Mult Veh - Crossing Path, Turn Into Path, Opp Dir'
	74 = 'Mult Veh - Crossing Path, Turn Into Path, Same Dir'
	75 = 'Mult Veh - Crossing Path, Turn Across Path, Opp Dir'
	76 = 'Mult Veh - Crossing Path, Turn Across Path, Same Dir'
	77 = 'Mult Veh - Crossing Path, Turn Across Path, Perp Dir'
	78 = 'Mult Veh - Crossing Path, Unknown Configuration'
	88 = 'Mult Veh - Vehicle Failure'
	100 = 'Mult Veh - Rear End, Struck'
	101 = 'Mult Veh - Opp Dir, Struck'
	102 = 'Mult Veh - Oth Veh Trav Wrong Dir'
	103 = 'Mult Veh - Oth Veh from Same Dir struck'
	104 = 'Mult Veh - Backing'
	200 = 'Multiple Scenarios'
	1000 = 'No Driver'
	.U = 'Unknown Preevent/Acctype';
run;
* larger crash scenario categories;
proc format;
VALUE crash_scen 
	0 = 'No Scenario'

	1 = 'Sing Veh - Departure'
	2 = 'Sing Veh - Control Loss'
	3 = 'Sing Veh - Animal in Road'
	4 = 'Sing Veh - Ped/Cyclist in Road'
	5 = 'Sing Veh - Object in Road'
	8 = 'Sing Veh - Other'

	20 = 'Mult Veh - Rear-end'
	21 = 'Mult Veh - Opp Dir, no turning'
	22 = 'Mult Veh - Same Dir, no turning'
	23 = 'Mult Veh - Control Loss'
	24 = 'Mult Veh - Object into Vehicle'
	25 = 'Mult Veh - Parked Car'

	30 = 'Mult Veh - Crossing Path, Both Straight'
	31 = 'Mult Veh - Turning Into/Across Path'

	41 = 'Mult Veh - Wrong Direction'
	42 = 'Mult Veh - U-turn'
	43 = 'Mult Veh - Backing'

	49 = 'Vehicle Failure'
	50 = 'Multiple/Conflicting Scenarios'
	100 = 'No Driver'
;
run;

* 1 - Determine a vehicle scenario for every vehicle;
data scenario_pre;
	format veh_scen veh_scen.;
	set vehicle_super;

	veh_scen = 0; * init;
	nveh = 1; * vehicle level counter (for tabulate);

	* SINGLE VEHICLE;
	if (vehforms = 1) then do;
		* fix parked car crashes;
		* in caseyear 2006 and earlier, parked car crashes were coded as single vehicle crashes;
		* the only way we can tell is using the old objcont1 codes of 70 and 71;
		if (objcont1 in(70 71) OR acctype = 11) then do;
			if (premove in(1,2,3,4,14)) then veh_scen = 32; * Mult Veh - parked car, lane keeping;
			else if (premove in(10,11,12)) then veh_scen = 33; * Mult Veh - parked car, turning;
			else if (premove = 17) then veh_scen = 34; * Mult Veh - parked car, avoidance;
			else if (premove in(6,15,16)) then veh_scen = 35; * Mult Veh - parked car, lane change;
			else veh_scen = 31; * Mult Veh - parked car, unknown maneuver;
		end;

		* first pass - critical pre-crash event;
		else if (preevent in(10,11,12,13,14) AND premove in(1,2,3,4,14))	AND not(acctype in(3, 8)) then veh_scen = 1; * Single Veh - no maneuver;
		else if (preevent in(10,11,12,13,14) AND premove in(6,15,16) AND not(acctype in (3, 8))) then veh_scen = 2; * Single Veh - lane change ;
		else if (preevent in(10,11,12,13,14) AND premove in(10,11,12) AND not(acctype in (3, 8))) then veh_scen = 15; * Single Veh - turning ;
		else if (preevent in(10,11,12,13,14) AND premove = 17) then veh_scen = 12; * Single Veh - avoidance maneuver ;
		else if (preevent in(10,11,12,13,14) AND premove = .U) then veh_scen = 3; * Single Veh - maneuver unknown ;
		else if (preevent in(10,11,12,13,14) AND premove = 13) then veh_scen = 4; * Single Veh - backing ;
		else if (preevent in(1,2,3,4) AND not(acctype in (3, 8))) then veh_scen = 5;   * Single Veh - veh failure ;
		else if ((5<=preevent<=9) AND (premove in(2,3,4,6,8,9,10,11,12,15,16,97)) AND not(acctype in(3, 8))) then veh_scen = 6;	* Single Veh - control loss, action;
		else if ((5<=preevent<=9) AND (premove in(1,14)) AND not(acctype in(3, 8))) then veh_scen = 7; * Single Veh - control loss, no action;
		else if ((5<=preevent<=9) AND (premove = 13)) then veh_scen = 8; * Single Veh - control loss, backing;
		else if ((5<=preevent<=9) AND (premove = .U)) then veh_scen = 9; * Single Veh - control loss, unkown maneuver;
		else if (50<=preevent<=78) then veh_scen = 12; * Single Veh - avoidance maneuver;
		else if ((preevent in(80)) AND (acctype in(12 13))) then veh_scen = 16; * Single Veh - Pedestrian in Road;
		else if ((preevent in(83)) AND (acctype in(12 13))) then veh_scen = 17; * Single Veh - Cyclist in Road;
		else if ((preevent in(90)) AND (acctype in(12 13))) then veh_scen = 18; * Single Veh - Object in Road;
		else if ((preevent in(87)) AND (acctype in(12 13))) then veh_scen = 19; * Single Veh - Animal in Road;

		* second pass - acctype;
		else if (acctype in(1,3,4,5,6,8,9,10) AND 50<=preevent<=92) then veh_scen = 12;  * Single Veh - avoidance maneuver;
		else if (acctype in(1,6,11,14) AND premove in(1,2,3,4,14)) then veh_scen = 1; * Single Veh - no maneuver;
		else if (acctype in(1,6,11,14) AND premove in(6,15,16)) then veh_scen = 2; * Single Veh- lane change ;
		else if (acctype in(1,6,11,14) AND premove in(10,11,12)) then veh_scen = 15; *Single Veh - lane change ;
		else if (acctype in(1,4,5,6,9,10,11,14) AND preevent in(1,2,3,4)) then veh_scen = 5; * Single Veh - veh failure;
		else if (acctype in(2,7) AND (premove in(2,3,4,6,8,9,10,11,12,15,16,97))) then veh_scen = 6; * Single Veh - control loss, action ;
		else if (acctype in(2,7) AND (premove in(1,14)) AND not(50<=preevent<=92)) then veh_scen = 7; * Single Veh - control loss, no action ;
		else if (acctype in(2,7) AND (premove = 13)) then veh_scen = 8; * Single Veh- control loss, backing ;
		else if (acctype in(2,7) AND (premove = .U)) then veh_scen = 9; * Single Veh - control loss, unkown maneuver ;
		else if (acctype in(2,7) AND (premove = 17)) then veh_scen = 12;  * Single Veh - avoidance maneuver;

		* third pass ;
		* the rest of these two acctypes have already been sorted ;
		else if (acctype in(3, 8)) then veh_scen = 12; * Single Veh - avoidance maneuver ;
		else if (acctype in(2, 7)) then veh_scen = 12; * Single Veh - avoidance maneuver ;
		else if (premove = 17) then veh_scen = 12; * Single Veh - avoidance maneuver ;

		* something else;
		else if (preevent in(10,11,12,13,14) OR acctype in(1,6,11,14)) then veh_scen = 3; * Single Veh- maneuver unknown  ;	
		else if (acctype in(4,5,9,10)) then veh_scen = 3; * Single Veh - maneuver unknown ;	
		else if (acctype = 15)	then do; 
			if (preevent = 87) then veh_scen = 19; 	* Single Veh  - Animal in Roadway ;
			else if (preevent = 90 AND objcont1 NE 77 AND e1 NE 77) then veh_scen = 18; * Single Veh - Object in Roadway (not a train); 
			else if (preevent = 80) then veh_scen = 16; * Single Veh - Pedestrian in road;
			else if (preevent = 83) then veh_scen = 17; * Single Veh - Cyclist in road;
			else if (objcont1 = 77 OR e1 = 77) then veh_scen = 20; * Single Veh  - Train ; 
			end;
		else if (acctype in(12,13)) then veh_scen = 19; 	* Single Veh  - Animal/Object in Roadway ;
		else if (objcont1 = 77 OR e1 = 77) then veh_scen = 20; * Single Veh  - Train ; 
		else veh_scen = 11; 	* Single Veh- Other;
	end;

	* MULTIPLE VEHILCES;
	else if (vehforms > 1) then do;

		* ------------------------------------;
		* first pass (preevent);
		* ------------------------------------;

		* rear-end and control loss;
		if ((preevent in(50,51,52))	and (premove in(1,2,3,4,14))) then veh_scen = 70; *	Mutl Veh - Rear End No Maneuver;
		else if ((preevent in(50,51,52)) and (premove in(6,7,10,11,12,13,15,16,17))) then veh_scen = 69; * Mult Veh - Rear End Maneuver;
		else if ((preevent = 53) AND (premove in(2,3,4,5,7))) then veh_scen = 100; * Mult Veh - Rear End, Struck;
		else if ((preevent in(1,2,3,4))) then veh_scen = 88; * Vehicle Failure;
		else if ((5<=preevent<=9) AND (premove in(1,14)) AND not(acctype in(3,8))) then veh_scen = 40; * Mult Veh - control loss, no veh action;
		else if ((5<=preevent<=9) AND (premove in(2,3,4,5,6,8,9,10,11,12,13,15,16,17))) then veh_scen = 41;	* Mult Veh - control loss, veh action;
		else if ((5<=preevent<=9)) then veh_scen = 42; * Mult Veh Depart - control loss, unknow veh action;

		* departure;
		else if ((preevent in(10,11)) AND (premove in(1,2,3,4,14)) AND (44<=acctype<=49)) then veh_scen = 24; * Mult Veh - same dir, drift out lane;
		else if ((preevent in(10,11)) AND (premove in(6,15,16)) AND (44<=acctype<=49 OR acctype in(70,72,74,75))) then veh_scen = 25; * Mult Veh - same dir, lane change;
		else if ((preevent in(51,52,53)) AND (premove in(6,15,16))) then veh_scen = 25; * Mult Veh - same dir, lane change;
		else if ((preevent in(10,11,51,52,53)) AND (premove in(10,11)) AND (44<=acctype<=49 OR acctype in(70,72,74,75))) then veh_scen = 76; * Mult Veh - Crossing Path, Turn Across Path, Same Dir; 
	
		else if ((preevent in(10,11,12,13,14)) AND (premove in(1,2,3,4,14)) AND (acctype in(50,52,53,64,66,67))) then veh_scen = 36; * Mult Veh - OD, depart lane, no maneuver;
		else if ((preevent in(10,11,12,13,14)) AND (acctype in(50,52,53,64,66,67)) AND (premove = 17)) then veh_scen = 37; * Mult Veh - OD, depart lane, avoidance maneuver;
		else if ((preevent in(10,11,12,13,14)) AND (acctype in(50,52,53,64,66,67)) AND (premove in(6,15,16))) then veh_scen = 38; * Mult Veh - OD, depart lane, lane change;

		else if ((preevent in(10,11,12,13,14)) AND (premove = 12)) then veh_scen = 29; * Mult Veh - U-turn;
		else if ((preevent in(15,53)) AND (premove = 12)) then veh_scen = 29; * Mult Veh - U-turn;
		else if ((preevent in(10,11)) AND (premove in(1,2,3,4,14)) AND (acctype in(20,24,28))) then veh_scen = 24; * Mult Veh - same dir, no maneuver; 
		else if ((preevent in(10,11)) AND (premove in(6,15,16)) AND (acctype in(20,24,28))) then veh_scen = 25; * Mult Veh - same dir, lane change; 
		else if ((preevent in(10,11,12,13,14)) AND (acctype = 11)) then do;
			if (premove in(1,2,3,4,14)) then  veh_scen = 32; * Mult Veh - parked car, no maneuver;
			else if (premove in(10,11)) then veh_scen = 33; * Mult Veh - parked car, turning;
			else if (premove = 17) then veh_scen = 34; * Mult Veh - parked car, avoidance;
			else if (premove in(6,15,16)) then veh_scen = 35; * Mult Veh - parked car, lane change;
			else veh_scen = 31; * Mult Veh - parked car, unknown maneuver;
			end;

		else if ((preevent = 54) AND (premove in(1,2,3,4,14))) then veh_scen = 102; * Mult Veh - Oth Vehicle Trav Wrong Dir;

		else if ((preevent in(10,11,12,13,14)) AND (acctype in(50,52,53,58,60,62,63,64,66,67))) then do; * Mult Veh - opposite direction;
			if (premove in(1,2,3,4,14)) then veh_scen = 36; * no maneuver;
			else if (premove = 17) then veh_scen = 37; * avoidance maneuver;
			else if (premove in(6,15,16)) then veh_scen = 38; * lane change;
			else if (premove = 11) then veh_scen = 75; * Mult Veh - Crossing Path, Turn Across Path, Opp Dir;
			else if (premove = 10) then veh_scen = 73; * Mult Veh - Crossing Path, Turn Into Path, Opp Dir;
			else if (premove = 12) then veh_scen = 29; * Mult Veh - U-turn;
			else veh_scen = 39; * Mult Veh - OD, depart lane, other maneuver;
			end;
		else if ((preevent in(10,11)) AND (premove in(6,15,16))) then veh_scen = 25;  * Mult Veh - same dir, lane change;
		else if ((preevent in(10,11)) AND (premove = 17)) then veh_scen = 23; * Mult Veh - same dir, avoidance;
		else if ((preevent in(10,11)) AND (premove in(1,2,3,4,14))) then do; 
			if (acctype in(20,24,28)) then veh_scen = 69; * Mult Veh - Rear End Maneuver;
			else veh_scen = 0; * Unknown Scenario;
			end;
		else if ((preevent in(12,13,14)) AND (premove in(1,14)) AND (acctype in(2,7))) then veh_scen = 40; * Mult Veh - control loss, no veh action;
		else if ((preevent in(12,13,14)) AND (premove in(2,3,4,5,6,8,9,10,11,12,13,15,16,17)) AND (acctype in(2,7))) then veh_scen = 41; * Mult Veh - control loss, veh action;
		else if ((preevent in(12,13,14)) AND (acctype in(2,7))) then veh_scen = 42; * Mult Veh - control loss, unknown veh action;
		else if ((preevent in(10,11)) AND (acctype in(46,47,70,73))) then veh_scen = 27; * Mult Veh - same dir, other/unknown maneuver;

		* crossing path;
		else if ((preevent in(17,66)) AND (86<=acctype<=91)) then veh_scen = 71; * Mult Veh - Crossing Path, Both Straight;
		else if ((preevent in(15,16,17)) AND (acctype in(80)) AND (premove in(4,10,11))) then veh_scen = 73; * Mult Veh - Crossing Path, Turn Into Path, Opp Dir; 
		else if ((preevent in(15,16,17)) AND (acctype in(76,78)) AND (premove in(4,10,11,14))) then veh_scen = 74; * Mult Veh - Crossing Path, Turn Into Path, Same Dir;
		else if ((preevent in(15,16,17,54,66)) AND (acctype in(68)) AND (premove in(4,10,11))) then veh_scen = 75; * Mult Veh - Crossing Path, Turn Across Path, Opp Dir; 
		else if ((preevent in(15,16,17)) AND (acctype in(70,72)) AND (premove in(4,10,11))) then veh_scen = 76; * Mult Veh - Crossing Path, Turn Across Path, Same Dir; 
		else if ((preevent in(15,16,17,66)) AND (acctype in(82)) AND (premove = 11)) then veh_scen = 77; * Mult Veh - Crossing Path, Turn Across Path, Perp Dir;
		
		* struck vehicle;
		else if ((acctype in(20, 24, 26)) AND (premove in(1,2,3,4,14))) then veh_scen = 70; * Mult Veh - Rear End No Maneuver ;
		else if ((acctype in(20, 24, 26)) AND (premove in(6,7,10,11,12,13,15,16,17))) then veh_scen = 69; * Mult Veh - Rear End Maneuver;
		else if (acctype = 92) then veh_scen = 104; * Mult Veh - Backing;

		* ------------------------------------;
		* Second Pass (acctype);
		* ------------------------------------;
		else if ((acctype in(46,47,48,49)) AND (premove in(1,2,3,4,14))) then do; 
			if (preevent in(60,61,62,63,64,74)) then veh_scen = 103; * Mult Veh - Oth Veh from Same Dir Struck;
			else veh_scen = 24; * Mult Veh - same dir, no maneuver ;
			end;
		else if ((acctype in(46,47,48,49,70,72)) AND (premove = 17)) then veh_scen = 23; * Mult Veh - same dir, avoidance maneuver ;
		else if ((acctype in(46,47,48,49,70,72)) AND (premove in(6,15,16))) then veh_scen = 25; * Mult Veh - same dir, lane change ;
		else if ((acctype in(46,47,48,49,70,72)) AND (premove in(10,11))) then veh_scen = 76; * Mult Veh - Crossing Path, Turn Across Path, Same Dir ;
		else if ((acctype in(46,47,48,49,70,72))) then veh_scen = 27; * Mult Veh - same dir, other/unknown maneuver ;
		else if (86<=acctype<=91 AND premove in(1,14)) then veh_scen = 71; * Mult Veh - Crossing Path, Both Straight; 
		else if ((acctype in(50,52,53,64,66,67))) then do; * opposite direction;
			if (preevent in(60,61,62,63)) then veh_scen = 101; * struck vehicle (so we can ID striking vehicle in crash); 
			else if (preevent = 15) then veh_scen = 73; * Crossing Path, Turn into Path, Opp Dir; 
			else if (premove in(1,2,3,4,14)) then veh_scen = 36; * Mult Veh - OD, depart lane, no maneuver ;
			else if (premove = 17) then veh_scen = 37; * Mult Veh - OD, depart lane, avoidance maneuver ;
			else if (premove in(6,15,16)) then veh_scen = 38; * Mult Veh - OD, depart lane, lane change ;
			else if (premove in(10,11)) then veh_scen = 75; * Mult Veh - Crossing Path, Turn Across Path, Opp Dir ;
			else if (premove = 12) then veh_scen = 29; * Mult Veh - U-turn ;
			else veh_scen = 39; * Mult Veh - OD, depart lane, other maneuver ;
			end;
		else if (acctype in(76,78) AND premove in(10,11)) then veh_scen = 74; * Mult Veh - Crossing path, Turn Into Path, Same Dir; 
		else if ((acctype = 68) and ((premove = 11) OR (preevent = 15))) then veh_scen = 75; *Mult Veh - Crossing Path, Turn Across Path, Opp Dir;
		else if ((acctype = 82) and ((premove = 11) OR (preevent = 15))) then veh_scen = 77; * Mult Veh - Crossing Path, Turn Across Path, Perp Dir;
		else if ((acctype in(84 85 74 75 90 91))) then veh_scen = 78; * Mult Veh - Crossing Path, Unknown Configuration;

		else if ((preevent in(10,11,12,13,14,15,16)) AND (acctype in(1,6,11)) AND (1<=e1<=30)) then do; 
			if (premove in(1,2,3,4,14)) then  veh_scen = 32; * Mult Veh - parked car, no maneuver;
			else if (premove in(10,11)) then veh_scen = 33; * Mult Veh - parked car, turning;
			else if (premove = 17) then veh_scen = 34; * Mult Veh - parked car, avoidance;
			else if (premove in(6,15,16)) then veh_scen = 35; * Mult Veh - parked car, lane change;
			else veh_scen = 31; * Mult Veh - parked car, unknown maneuver;
			end;
		else if ((preevent in(10,11,12,13,14)) AND (acctype in(1,6,14,15,16)) AND (premove in(1,2,3,4,14))) then veh_scen = 50; * Mult Veh - Roadside object into Vehicle, no maneuver;
		else if ((preevent in(10,11,12,13,14)) AND (acctype in(1,6,14,15,16)) AND (premove in(6,10,11,12,15,16))) then veh_scen = 51; * Roadside object into Vehicle, maneuver; 
		else if (premove = 12) then veh_scen = 29; * Mult Veh - U-Turn; 
	end;
	
	if (premove = 0) then veh_scen = 1000; * No Driver;
	if (preevent = .U AND acctype = .U) then veh_scen = .U; * Unknown Preevent/Acctype;

	premoved = propcase(put(premove, premove.));
	preeventd = propcase(put(preevent, preevent.));
run;

* 2 - Take only the vehicle(s) invovled in first harmful event;
proc sort data=scenario_pre out=spsort;
	by psu caseno vehno;
run;
data spsortv;
	format v1num 8. v2num 8.;
	set spsort;
	/* grab the first vehicle-to-vehicle crash */
	if (first_vehs = '99') then v1num = 1; * no vehicle-to-vehicle collision, single vehicle;
	else do;
		v1num = input(substr(first_vehs, 1, 1), 8.);
		v2num = input(substr(first_vehs, 2, 1), 8.);
	end;
run;
data caselist1; * every vehicle involved in first vehicle-to-vehicle crash (if there is one, else it's a single-vehicle crash);
	format v1_veh_scen veh_scen.;
	set spsortv;
	by psu caseno vehno;

	if (vehno = v1num);

	v1_veh_scen = veh_scen;
	v1_vehno = vehno;
	v1_e1 = e1;
	v1_preevent = preevent;
	v1_premove = premove;
	v1_acctype = acctype;
	keep psu caseno v1_veh_scen v1_vehno v1_e1
		v1_preevent v1_premove v1_acctype ratwgt vehforms v1num v2num 
		nveh;
run;
data caselist2; * every vehicle;
	format v2_veh_scen veh_scen.;
	set spsortv;

	if (vehno = v2num);

	v2_veh_scen = veh_scen;
	v2_vehno = vehno;
	v2_e1 = e1;
	v2_preevent = preevent;
	v2_premove = premove;
	v2_acctype = acctype;

	keep psu caseno v2_vehno v2_veh_scen v2_vehno v2_e1
		v2_preevent v2_premove v2_acctype ratwgt vehforms 
		nveh;   
run;
proc sort data=caselist2; by psu caseno;
proc sort data=caselist1; by psu caseno;
data cases_matched; * match each vehicle with it's file from the first struck;
	merge caselist1 (in=clist) caselist2 (in=othveh);
	by psu caseno;
run;

* 3 - Determine a crash scenario for the crash using first two vehicles;
data cases_rdscen;
	format rd_crash veh_scen. crash_scen crash_scen.;
	set cases_matched;
	by psu caseno;

	if (v1_veh_scen in(., .U)) then crashn = 2; * missing, use other scenario, note this should never happen;
	else if (v2_veh_scen in(., .U)) then do; 
		if (v1_veh_scen = 40) then rd_crash = 7; * no vehicle to vehicle contact - recode as single vehicle crash (often when other vehicle is damaged by debris);
		else if (v1_veh_scen = 41) then rd_crash = 6;
		else if (v1_veh_scen = 50) then rd_crash = 1;
		else if (v1_veh_scen = 51) then rd_crash = 3;

		crashn = 1; * missing, use other scenario;
		end;
	else if ((v1_veh_scen = 100 and v2_veh_scen = 100) OR (v1_veh_scen = 101 and v2_veh_scen = 101)) then do;	* both struck veh...messed up;
		rd_crash = 0;
		rd_vehno = 999;
		rd_e1 = 999;
	end;
	else if (v1_veh_scen = v2_veh_scen) then do; * same, probably LTAP, SCP, or vehicles turning;
		rd_crash = v1_veh_scen;
		rd_vehno = 999;
		rd_e1 = 999;
	end;
	else if (v1_veh_scen in(69, 70)) AND (v2_veh_scen in(69, 70)) then do; * both coded as (different) rear-end scenarios - can't tell which is striking;
		rd_vehno = 999;
		rd_e1 = 999;
		rd_crash = v1_veh_scen;
		end;
	else if (v1_veh_scen in(69, 70)) then crashn = 1; * rear-end;
	else if (v2_veh_scen in(69, 70)) then crashn = 2; * rear-end;
	else if (v1_veh_scen = 100) then do;	* rear-end struck; 
		if (v2_veh_scen in(40, 41)) then crashn = 2; * control loss takes precedence;
		else do;
			rd_crash = 70;
			crashn = 2;
			rd_e1 = v2_e1;
		end;
	end;
	else if (v2_veh_scen = 100) then do;	* rear-end struck;
		if (v1_veh_scen in(40, 41)) then crashn = 1; * control loss takes precedence;
		else do;
			rd_crash = 70;
			crashn = 1;
			rd_e1 = v2_e1;
		end;
	end;
	else if (v1_veh_scen = 101 AND v2_veh_scen NE 0 and v2_veh_scen NE 1000)  then crashn = 2; * opposite direction struck vehicle;
	else if (v2_veh_scen = 101 AND v1_veh_scen NE 0 and v1_veh_scen NE 1000) then crashn = 1;
	else if (v1_veh_scen = 102 AND v2_veh_scen = 102) then do; * other vehicle traveling in opposite direciton;
		rd_crash = 102;
		rd_vehno = 999;
		rd_e1 = 999;
		end;
	else if (v1_veh_scen = 102 AND v2_veh_scen NE 102) then crashn = 2; * use other scenario; 
	else if (v2_veh_scen = 102 AND v1_veh_scen NE 102) then crashn = 1; * use other scenario; 
	else if (v1_veh_scen = 103 AND v2_veh_scen NE 0 AND v2_veh_scen NE 1000) then crashn = 2; * same direction struck veh; 
	else if (v2_veh_scen = 103 AND v1_veh_scen NE 0 AND v1_veh_scen NE 1000) then crashn = 1; 
	else if (v1_veh_scen = 1000 and v2_veh_scen NE 0) then crashn = 2; * don't use the no driver car;
	else if (v2_veh_scen = 1000 and v1_veh_scen NE 0) then crashn = 1; * don't use the no driver car;
	else if ((v1_veh_scen = 1000 and v2_veh_scen = 0) or (v2_veh_scen = 1000 and v1_veh_scen = 0)) then do;	* no scenario;
		rd_crash = 0;
		rd_vehno = 999;
		rd_e1 = 999;
	end;
	else if (v1_veh_scen in(40,41,42)) then crashn = 1; * control loss in the crash trumps other scenarios;
	else if(v2_veh_scen in(40,41,42)) then crashn = 2; * control loss in the crash trumps other scenarios;
	else if (v1_veh_scen in(5 88)) then crashn = 1; * veh failure in the crash trumps other scenarios;
	else if (v2_veh_scen in(5 88)) then crashn = 2; * veh failure in the crash trumps other scenarios;
	else if (v1_veh_scen = 0) then crashn = 2; * normal case, no scenario;
	else if (v2_veh_scen = 0) then crashn = 1; * normal case, no scenario;
	else do; * 2 scenarios;
		if (v1_veh_scen = 73) and not(v2_veh_scen in(0 1000)) then crashn = 2;
		else if (v2_veh_scen = 73) and not(v1_veh_scen in(0 1000)) then crashn = 1;
		else do;
			rd_crash = 200;
			rd_vehno = 999;
			rd_e1 = 999;
			end;
	end;

	if (crashn = 1) then do;
		if (rd_crash = .) then 
			rd_crash = v1_veh_scen;
		rd_premove = v1_premove;
		rd_acctype = v1_acctype;
		rd_preevent = v1_preevent;
		rd_vehno = v1_vehno;
		rd_e1 = v1_e1;
	end;
	else if (crashn = 2) then do;
		if (rd_crash = .) then
			rd_crash = v2_veh_scen;
		rd_vehno = v2_vehno;
		rd_premove = v2_premove;
		rd_acctype = v2_acctype;
		rd_preevent = v2_preevent;
		rd_e1 = v2_e1;
	end;

	* crash_scen;
	if (rd_crash = 0) then crash_scen = 0; *No Scenario;
	else if (rd_crash in(1,2,15,12,3,4)) then crash_scen = 1; *Sing Veh - Departure;
	else if (rd_crash in(6,7,8,9)) then crash_scen = 2; *Sing Veh - Control Loss;
	else if (rd_crash in(19)) then crash_scen = 3; *Sing Veh - Animal;
	else if (rd_crash in(16,17)) then crash_scen = 4; *Sing Veh - Ped/Cyclist in Road;
	else if (rd_crash in(18)) then crash_scen = 5; *Sing Veh - Object in Road;
	else if (rd_crash in(11, 20)) then crash_scen = 8; *Sing Veh - Other (includes train);
	
	else if (rd_crash in(69,70)) then crash_scen = 20; * Mult Veh - Rear-end;
	else if (rd_crash in(36,37,38,39)) then crash_scen = 21; * Mult Veh - Opp Dir, no turning;
	else if (rd_crash in(23,24,25,27)) then crash_scen = 22; * Mult Veh - Same Dir, no turning;
	else if (rd_crash in(40,41,42)) then crash_scen = 23; * Mult Veh - Control Loss;
	else if (rd_crash in(50,51)) then crash_scen = 24; * Mult Veh - Object into Vehicle;
	else if (rd_crash in(31,32,33,34,35)) then crash_scen = 25; * Mult Veh - Parked Car;

	else if (rd_crash = 71) then crash_scen = 30; * Mult Veh - Crossing Path, Both Straight;
	else if (rd_crash in(73,74,75,76,77,78)) then crash_scen = 31; * Mult Veh - Turning Into/Across Path;

	else if (rd_crash in(102)) then crash_scen = 41; * Mult Veh - Wrong Direction;
	else if (rd_crash in(29)) then crash_scen = 42; * Mult Veh - U-turn;
	else if (rd_crash in(104)) then crash_scen = 43; * Mult Veh - Backing;
	else if (rd_crash in(5, 88)) then crash_scen = 49; *  Vehicle Failure;
	else if (rd_crash in(101,103,200)) then crash_scen = 50; * Multiple/Conflicting Scenarios;
	else if (rd_crash in(1000)) then crash_scen = 100; * No Driver;

	* Single or Multi-vehicle scenario;
	if (rd_crash = 0 OR rd_crash = 1000) then single_or_mult = ''; * unknown;
	else if (1<=crash_scen<=8) then single_or_mult = 'Single';
	else if (20<=crash_scen<=80) then single_or_mult = 'Mult';
run;

* Check with Appendix C Table C1 in Paper;
proc freq data=cases_rdscen order=freq;
	tables crash_scen;
run;
